/**
 * @function doReset
 */
"use strict";

const Promise = require('apemanpromise');

/** @lends doReset */
function doReset(UserRecover, digest, token, password) {
    let User = UserRecover.getRef('user').model;
    return UserRecover
        .findOne({
            where: {
                recover_token: token,
                recovered: false
            }
        })
        .then(recover => {
            if (!recover) {
                return null;
            }
            let expired = new Date() < new Date(recover.expire_at);
            if (expired) {
                return null;
            }
            return User
                .findById(recover.user_id, {
                    include: [User.getRef('sign')]
                })
                .then(user =>
                    Promise.all(
                        user.sign.map(sign =>
                            digest(password || sign.password, sign.password_salt)
                                .then(hash =>
                                    sign.update({
                                        password_hash: hash,
                                        signmod_at: new Date()
                                    })
                                )
                        )
                    )
                )
                .then(() => recover)
        })
        .then(recover =>
            recover && recover.update({
                recovered: true,
                recovered_at: new Date()
            })
        )
}

module.exports = doReset;
